#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <netinet/in.h>
#include <netdb.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <time.h>
#include "enum.h"

enum_target_t	*enum_anchor;

void enum_free() {
    enum_target_t	*c,*c2;

    if ((c=enum_anchor)==NULL) return;
    while (c!=NULL) {
	c2=c;
	c=c->next;
	free(c2);
    }
}

int enum_known(struct in_addr *addr) {
    enum_target_t	*c;

    if ((c=enum_anchor)==NULL) return 0;
    while (c!=NULL) {
	if (memcmp(&(c->addr),addr,4)==0) return 1;
	c=c->next;
    }
    return 0;
}

int enum_init( enum_target_t **c ) {

    if ((*c=enum_anchor)==NULL) return -1;

return 0;
}

int enum_next( enum_target_t **c ) {

    *c=(*c)->next;

    if (*c==NULL) return -1;

return 0;
}

int enum_print() {
    enum_target_t	*c;

    if ((c=enum_anchor)==NULL) return 0;
    while (c!=NULL) {
	printf("%s\n",inet_ntoa(c->addr));
	c=c->next;
    }
    return 0;
}

/* returns the number of valid targets or -1 on error 
 *
 * dest		is the destination in one of the following notations
 * 		- FQDN (www.targets.com)
 * 		- IP-Address (192.168.1.1)
 * 		- IP/Netmask (192.168.1.0/255.255.255.0)
 * 		- IP/BitMask (192.168.1.0/24)
 */
int enumerate(char *dest) {
    int			sfd;
    enum_target_t	*current,*new;
    u_int32_t		tnet,tnet2,l;
    char		*tp,*tp2;
    struct in_addr	n,m,mm;
    unsigned long int	t1;
    unsigned int	number_of_targets=0;

    enum_free();
    enum_anchor=NULL;

    /* if the destination contains a / it is assumed to be a network/mask 
     * notation - eiter dotted mask or bits */
    if (!strchr(dest,'/')) {
	if (inet_aton(dest,&n)==0) {
	    struct hostent	*hd;
	    /* destination not IP address - try to resolve it */
            if ((hd=gethostbyname(dest))==NULL) {
                return (-1);
            } else {
                bcopy(hd->h_addr,(char *)&(n),hd->h_length);
            }
	}
	/* this is just one */
	enum_anchor=malloc(sizeof(enum_target_t));
	memcpy(&(enum_anchor->addr),&(n),sizeof(struct in_addr));
	number_of_targets=1;
	
    } else {
	/* assumption: these are mutiple targets */
	u_int32_t	q = 0xFFFFFFFF;
	int		lx1,lx2;

	/* multiple targets ....
	 * first, we have to figure out where ... */
	tp=malloc(strlen(dest)+1);
	strcpy(tp,dest);
	tp2=strchr(tp,'/');
	tp2[0]='\0';
	tp2++;

	if (!strchr(tp2,'.')) {
	    lx1=atoi(tp2);
	    for (lx2=32;lx2>lx1;lx2--) q=q<<1; 
	    q=htonl(q);
	    memcpy(&(m.s_addr),&q,4);
	} else {
	    if (inet_aton(tp2,&m)==0) {
		return (-1);
	    }
	}
	memcpy(&mm,&m,sizeof(m));

	/* network part must resulve as dotted */
	if (inet_aton(tp,&n)==0) {
	    fprintf(stderr,"%s seems to be a stange network address\n",tp);
	    return (-1);
	}

	/* calculate first and last address. */
	tnet = ntohl(  n.s_addr&m.s_addr  );
	tnet2= ntohl(  (m.s_addr^0xFFFFFFFF)|n.s_addr  );

	/* add the records to the target list */
	t1=(unsigned long int)time(NULL);
	l=tnet;
	current=NULL;

	    /* no network activity - just add them */
	    for (l=tnet;l<=tnet2;l++) {
		n.s_addr=htonl(l);
		new=current;
		current=malloc(sizeof(enum_target_t));
		memcpy(&(current->addr),
			&(n),
			sizeof(struct in_addr));
		if (new==NULL) { new=current; } 
		else { new->next=current; }
		if (enum_anchor==NULL) { 
		    enum_anchor=current; 
		}
		number_of_targets++;
	    }
	}
	close(sfd);

    if (enum_anchor==NULL) return(-1);

    return number_of_targets;
}
