/* UDP Scanner (protocol dependent packets sender)
        By: fryxar
        Compile: gcc scanudp.c -o scanudp
*/
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/select.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <unistd.h>
#include <signal.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <sys/time.h>
#include <netdb.h>

#include "enum.h"

#define VERSION "2.0"
#define MAXBUF 1400
#define DEFAULT_TIMEOUT 10

typedef struct config {
        int timeout;
        int bandwidth;
} config;

typedef struct port_t {
        u_int16_t number;			// Number of udp port
        char *name;					// Name of udp port
        char *outstring;			// String to send (protocol dependent)
        int outstringlen;			// Len above
} scan;

// Port scanning probes definitions (protocol dependent)
struct port_t             port[] = {

        7, "echo", 
                "probe", 5,

        13, "daytime", 
                "\x0a", 1,

        19, "chargen", 
                "\x0a", 1,

        // dig @ip localhost A
        53, "dns", 
                "\x68\x6c\x01\x00\x00\x01\x00\x00\x00\x00\x00\x00\x09\x6c\x6f\x63\x61\x6c\x68\x6f\x73\x74\x00\x00\x01\x00\x01", 27,

        // echo "get a" | tftp ip
        69, "tftp", 
                "\x00\x01\x61\x00\x6e\x65\x74\x61\x73\x63\x69\x69\x00", 13,

        // ntpq -p ip
        123, "ntp", 
                "\x16\x01\x00\x01\x00\x00\x00\x00\x00\x00\x00\x00", 13,

        // nbtstat -A ip
        137, "ns-netbios", 
                "\x98\x38\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x20\x43\x4b\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x00\x00\x21\00\01", 50,

        // snmpwalk ip ILMI
        161, "snmp(ILMI)", 
                "\x30\x24\x02\x01\x00\x04\x04\x49\x4c\x4d\x49\xa1\x19\x02\x04\x18\x39\x99\xcd\x02\x01\x00\x02\x01\x00\x30\x0b\x30\x09\x06\x05\x2b\x06\x01\x02\x01\x05\x00", 38,

        // snmpwalk ip public
        161, "snmp(public)", 
                "\x30\x26\x02\x01\x00\x04\x06\x70\x75\x62\x6c\x69\x63\xa1\x19\x02\x04\x2c\x60\x2d\xb6\x02\x01\x00\x02\x01\x00\x30\x0b\x30\x09\x06\x05\x2b\x06\x01\x02\x01\x05\x00", 40,

        0, NULL, NULL, 0,
};

void usage(char *program) {
int i;

        fprintf( stderr,
        "%s v"VERSION" -   by: Fryxar\n"
        "usage: %s [options] <host>\n\n"
        "options:\n"
        " -t <timeout>     Set port scanning timeout\n"
        " -b <bps>         Set max bandwidth\n"
        " -v               Verbose\n"
        "\nSupported protocol:\n"
        , program, program);

        for( i=0; port[i].number; i++)
                fprintf( stderr, "%s ", port[i].name );
        fprintf( stderr, "\n\n" );

exit(-1);
}

struct config conf;

void ListenPackets( int fd, struct timeval tv ) {
char                    buf[MAXBUF];
fd_set                  fdsr;
socklen_t               socklen;
struct sockaddr_in      src_addr;

   FD_ZERO( &fdsr );
   FD_SET( fd, &fdsr );

   if( select( fd+1, &fdsr, NULL, NULL, &tv ) < 0 ) {
        perror("select: ");
        exit(-1);
   }

   if( !FD_ISSET(fd, &fdsr) ) return;
   
   socklen = sizeof(src_addr);
   if( recvfrom(fd, buf, MAXBUF, 0, 
             (struct sockaddr *)&src_addr, &socklen) <= 0 ) {
                  close( fd );
                  exit -1;
   }

   printf( "%s\t%d\n", inet_ntoa(src_addr.sin_addr), htons(src_addr.sin_port) );
}

int main(int argc, char *argv[]) {
char                    buf[MAXBUF];
char                    opt, *host;
struct timeval          tv;
struct sockaddr_in      dst_addr, src_addr;
struct  hostent         *he;
time_t                  t_start, t_now;
int                     fd, i, verbose;
enum_target_t           *ip;

// Set defaults
        conf.timeout = DEFAULT_TIMEOUT;           /* Default timeout */
        verbose = 0;                              /* No verbose */
        conf.bandwidth = 128000;                  /* 128 Kbs */

        if(argc < 2) usage( argv[0] );

        while((opt = getopt(argc, argv, "vt:b:")) != -1) {
                switch(opt) {
                        case 't':
                                if(strlen(optarg) == 0) usage(argv[0]);
                                conf.timeout = atoi(optarg);
                                break;
                        case 'v':
                                verbose = 1;
                                break;
                        case 'b':
                                conf.bandwidth = atoi(optarg);
                                if(conf.bandwidth == 0) usage(argv[0]);
                                break;
                        default:
                                usage(argv[0]);
                                break;
                }
        }

        tv.tv_sec = 0;
        tv.tv_usec = 800000000 / conf.bandwidth;

        if (tv.tv_usec >= 1000000) {
            tv.tv_sec = tv.tv_usec / 1000000;
            tv.tv_usec = tv.tv_usec % 1000000;
        }

        if( enumerate(argv[argc-1]) < 0 ) {
            printf("error in destination\n");
            exit -1;
        }

        if((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
                        perror("socket: ");
                        exit(2);
        }

       src_addr.sin_family = AF_INET;
       src_addr.sin_addr.s_addr = htonl(INADDR_ANY);
       src_addr.sin_port = rand();


       if( bind(fd, (struct sockaddr *)&src_addr, sizeof(src_addr)) < 0 ) {
             perror("bind: ");
             close(fd);
             exit(-1);
       }

       dst_addr.sin_family = AF_INET;

       enum_init( &ip );

       while( ip != NULL ) {
           if( verbose )
                 printf( "Scanning IP %s\n", inet_ntoa(ip->addr) );

           for( i=0; port[i].number; i++ ) {

                memcpy( (char *)&dst_addr.sin_addr, (char *)&(ip->addr), sizeof(ip->addr) );
                dst_addr.sin_port =  htons(port[i].number);

                memcpy( buf, port[i].outstring, port[i].outstringlen );

                if(sendto(fd, buf, port[i].outstringlen, 0, 
                    (struct sockaddr *)&dst_addr, sizeof(dst_addr)) < 0) {
                        perror("sendto: ");
                        close(fd);
                        exit(-1);
                }

                ListenPackets( fd, tv );
           }

            enum_next( &ip );
       }

       time(&t_start);
       while( 1 ) {
            time( &t_now );
            if( t_now-t_start >= conf.timeout ) break;
            tv.tv_sec  = conf.timeout-(t_now-t_start);
            tv.tv_usec = 0;
            ListenPackets( fd, tv );
       }

        close( fd );

exit(0);
}
